#!/usr/bin/perl -w

######################################################
#
#	epc wrapper
#
#	This script explore server performance between 2 configurations
#	and produce a normalized TCO and a normalized environmental impact.
#
#	The results are printed in a file <output_file_name>.txt
#	and the graph is plot in a file <output_file_name>.eps
#
#	How to use it:
#	./epc_performance.pl	<dc config file reference>			datacenter configuration file of the reference configuration
#							<resource config file reference>	reference resource configuration file
#							<dc config file>					datacenter configuration file (sweep on this configuration)
#							<resource config file>				resource configuration file (sweep on this configuration)
#							<output_file_name>					filename of the output file without extension (the extensions are .txt and .eps)
#							[nognuplot]							Optional: put "nognuplot" if you do not want to get the graph
#
#	Remarks:
#		The number of reference server is assumed to be 50000
#
#	Example of command:
#		 ./epc_performance.pl ../template/dc_template.params ../template/r_template.params ../template/dc2_template.params ../template/r2_template.params epc
#
######################################################


use strict;
use File::Copy;

use lib "../lib/"; 
use helper;

######################################################
#
# Get input parameters
#
######################################################

my $argv_size = scalar @ARGV;
if ($argv_size < 5 )
{
	print "usage: ./epc_performance.pl <dc config file reference> <resource config file reference> <dc config file> <resource config file> <output_file_name> [nognuplot]\n";
	exit(0);
}

my $dc_filename_reference=$ARGV[0];
my $resource_filename_reference=$ARGV[1];

my $dc_filename_sweep=$ARGV[2];
my $resource_filename_sweep=$ARGV[3];

my $filename_output= $ARGV[4];

my $gnuplot=1;

if(defined($ARGV[5]))
{		
	if($ARGV[5] eq "nognuplot")
	{
		$gnuplot=0;
	}
	else{print ("Error: parameter $ARGV[5] not correctly defined\n");exit(0);}
}




######################################################
#
# General Initialization
#
######################################################

#create the helper
my $helper = helper->new("");

#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

#output file's name  without extension
my $filename=$filename_output;

#legende
my $legende="#epc \t normalized TCO \t normalized env impact\n";

#open outputfile and print the legende
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;
print(OUTPUTFILE "$legende");


######################################################
#
# Specific Initialization
#
######################################################

#number of servers (reference)
my $nbserver_ref=50000;


############################################################
#get the reference tco value and env impact reference value
############################################################

#generate the reference configuration files
my $ptr_dc=$helper->parserDCFile("$dc_filename_reference");
$helper->createParameterFile("$currentdir/dc.params",$ptr_dc);

my $ptr_resource_ref=$helper->parserResourceFile("$resource_filename_reference");
my %resource_ref=%{$ptr_resource_ref};
$resource_ref{n_srvmodulesreq}=$nbserver_ref;
$helper->createParameterFile("$currentdir/s.params",\%resource_ref);

#call the eetco kernel and keep the output
$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/ref.res");

#get the TCO  & co2 values
my $ptr_res=$helper->parserResultKernel("$currentdir/output/ref.res");
my %res=%{$ptr_res};
my $tco_ref=$res{tco};
my $env_impact_ref=$res{co2peryr};

############################################################
#get the configuration to do the sweep
############################################################

$ptr_dc=$helper->parserDCFile("$dc_filename_sweep");
$helper->createParameterFile("$currentdir/dc.params",$ptr_dc);

my $ptr_resource_sweep=$helper->parserResourceFile("$resource_filename_sweep");
my %resource_sweep=%{$ptr_resource_sweep};


############################################################
# main exploration loop
############################################################
foreach my $epc(1..6)
{
	my $nbserver=$nbserver_ref*$epc;
	
	#generate the current configuration files
	$resource_sweep{n_srvmodulesreq}=$nbserver;
	$helper->createParameterFile("$currentdir/s.params",\%resource_sweep);
	
	#call the eetco kernel and keep the output
	$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/$epc.value");
	
	#get the TCO  & co2 values
	$ptr_res=$helper->parserResultKernel("$currentdir/output/$epc.value");
	%res=%{$ptr_res};
	
	my $tco=$res{tco};
	my $env_impact=$res{co2peryr};

	my $normalized_tco=$tco/$tco_ref;
	my $normalized_env_impact=$env_impact/$env_impact_ref;
	
	print(OUTPUTFILE "$epc\t$normalized_tco\t$normalized_env_impact\n");
}

close OUTPUTFILE;

#keep the output file in the current directory
copy("$currentdir/output/$filename.txt","$currentdir/$filename.txt") or die "Copy failed: $!";


####################################################
# gnulpot call
####################################################
if($gnuplot==1)
{
	my @curves_names=("Normalized TCO","Normalized CO2");
	$helper->makeGnuplotCurves2D("$currentdir/$filename.txt","$currentdir","$filename","Performance (epc)","Normalized (TCO or CO2)",\@curves_names);
}

####################################################
# clean
####################################################
$helper->deleteFile("$currentdir/dc.params");
$helper->deleteFile("$currentdir/s.params");
$helper->deleteFileInDir("$currentdir/output/");
